@@ -209,65 +209,70 @@ limitsFromLimitlist <-
         ## class.
 
 
+        #if (!have.lim)
+        ## always calculate the limits from prepanel first:
+
+        ## should check that all classes are the same. How ? What
+        ## about NA's ? Arrgh!
+
+        ## to handle NA's, how about:
+
+        all.na <- unlist(lapply(limitlist, function(x) all(is.na(x))))
+        class.lim <- ## retain non-NA limitlists only
+            lapply(limitlist[!all.na], class)
+        ## class.lim is a list now, may be length 0
+        limits <- unlist(limitlist) ## loses the class attribute
+
+        ## if (length(limits) > 0)
+        if (sum(!is.na(limits)) > 0)
+        {
+            if (is.character(limits))
+            {
+                limits <- unique(limits[!is.na(limits)])
+                slicelen <- diff(extend.limits(limits, axs = axs))
+            }
+            else ## if (is.numeric(limits)) # or dates etc
+            {
+                limits <-
+                    extend.limits(range(as.numeric(limits), finite = TRUE),
+                                  axs = axs)
+                slicelen <- diff(range(limits, finite = TRUE))
+            }
+
+            ## hopefully put back appropriate class of limits:
+            ## FIXME: date changes may have messed this up
+
+            if (length(class.lim) > 0)
+                class(limits) <-
+                    if (all(class.lim[[1]] == "integer"))
+                        "numeric" else class.lim[[1]]
+
+            ## (have to handle "integer" specially, since variable
+            ## specifications like 1:10 are rather common, and
+            ## class() <- "integer" would turn the limits into
+            ## integers)
+        }
+        else
+        {
+            limits <- c(0,1)
+            slicelen <- 1
+        }
+
         if (have.lim)
         {
             if (is.list(lim))
                 stop("limits cannot be a list when relation = same")
+            old.limits <- limits
             limits <- lim
+            ## lim overrides prepanel except NAs
+            if (!is.character(limits))
+                limits[is.na(limits)] <- old.limits[is.na(limits)]
             slicelen <-
                 ## this no longer works for dates (R 2.6)
 ##                 if (is.numeric(lim)) diff(range(lim))
 ##                 else length(lim) + 2
-                if (is.character(lim)) length(lim) + 2
-                else diff(range(as.numeric(lim)))
-        }
-        else
-        {
-            ## should check that all classes are the same. How ? What
-            ## about NA's ? Arrgh!
-
-            ## to handle NA's, how about:
-
-            all.na <- unlist(lapply(limitlist, function(x) all(is.na(x))))
-            class.lim <- ## retain non-NA limitlists only
-                lapply(limitlist[!all.na], class)
-            ## class.lim is a list now, may be length 0
-            limits <- unlist(limitlist) ## loses the class attribute
-
-            ## if (length(limits) > 0)
-            if (sum(!is.na(limits)) > 0)
-            {
-                if (is.character(limits))
-                {
-                    limits <- unique(limits[!is.na(limits)])
-                    slicelen <- length(limits) + 2
-                }
-                else ## if (is.numeric(limits)) # or dates etc
-                {
-                    limits <-
-                        extend.limits(range(as.numeric(limits), finite = TRUE),
-                                      axs = axs)
-                    slicelen <- diff(range(limits, finite = TRUE))
-                }
-
-                ## hopefully put back appropriate class of limits:
-                ## FIXME: date changes may have messed this up
-
-                if (length(class.lim) > 0)
-                    class(limits) <-
-                        if (all(class.lim[[1]] == "integer"))
-                            "numeric" else class.lim[[1]]
-
-                ## (have to handle "integer" specially, since variable
-                ## specifications like 1:10 are rather common, and
-                ## class() <- "integer" would turn the limits into
-                ## integers)
-            }
-            else
-            {
-                limits <- c(0,1)
-                slicelen <- 1
-            }
+                if (is.character(limits)) length(limits) + 2
+                else diff(range(as.numeric(limits)))
         }
         ans <- list(limits = limits, slicelen = slicelen)
     }
@@ -350,6 +355,16 @@ limitsFromLimitlist <-
             limitlist[id] <- lim
             which.null <- sapply(limitlist, is.null)
             limitlist[which.null] <- old.limitlist[which.null]
+
+            ## lim overrides prepanel except NAs
+            for (i in seq_along(limitlist))
+            {
+                if (!is.character(limitlist[[i]]))
+                {
+                    isna <- is.na(limitlist[[i]])
+                    limitlist[[i]][isna] <- old.limitlist[[i]][isna]
+                }
+            }
         }
         for (i in seq_along(limitlist))
         {
@@ -412,6 +427,13 @@ limits.and.aspect <-
                 prenames <- names(formals(prepanel))
                 if (!("..." %in% prenames)) pargs <- pargs[intersect(names(pargs), prenames)]
                 pretem <- do.call("prepanel", pargs)
+                ## prepanel() over-rides defaults except NAs - e.g. ylim = c(0, NA)
+                if (!is.null(pretem$xlim) && !is.character(pretem$xlim))
+                    if (any(isna <- is.na(pretem$xlim)))
+                        pretem$xlim[isna] <- tem$xlim[isna]
+                if (!is.null(pretem$ylim) && !is.character(pretem$ylim))
+                    if (any(isna <- is.na(pretem$ylim)))
+                        pretem$ylim[isna] <- tem$ylim[isna]
                 tem <- updateList(tem, pretem)
                 ## tem[names(pretem)] <- pretem
             }
